import { Graph } from 'graphlibrary';
import d3Renderer from 'dagre-d3-renderer';
import { FlowElement } from './FlowElement';
import { select } from 'd3';
var FlowChart = /** @class */ (function () {
    function FlowChart(options) {
        this.options = {
            direction: 'LR'
        };
        this.elements = [];
        this.options = Object.assign(this.options, options);
    }
    FlowChart.prototype.addElement = function (id, options) {
        var el = new FlowElement(id, options);
        this.elements.push(el);
        return el;
    };
    FlowChart.prototype.render = function (element) {
        var svg = select(element)
            .append('svg')
            .attr('id', 'f' + element.id)
            .attr('xmlns', 'http://www.w3.org/2000/svg');
        var svgGroup = svg.append('g');
        // Create the input mermaid.graph
        var g = new Graph({
            multigraph: true,
            compound: true
        })
            .setGraph({
            rankdir: this.options.direction,
            marginx: 20,
            marginy: 20
        })
            .setDefaultEdgeLabel(function () {
            return {};
        });
        // first create all nodes
        for (var i in this.elements) {
            var el = this.elements[i];
            var elData = {
                label: el.id
            };
            if (el.options && el.options.label) {
                elData.label = el.options.label;
            }
            g.setNode(el.id, elData);
            var node = g.node(el.id);
            // apply some styles
            node.rx = node.ry = 5;
            // now create all edges
            for (var k in el.edges) {
                var edge = el.edges[k];
                var edgeData = {};
                if (edge.options && edge.options.label) {
                    edgeData.label = edge.options.label;
                }
                g.setEdge(el.id, edge.otherId, edgeData);
            }
        }
        var render = new d3Renderer.render();
        var e = select('#f' + element.id + ' g');
        render(e, g);
        var svgElement = document.getElementById('f' + element.id);
        // now add the listeners after render
        e.selectAll('g.node')
            .each(function (v) {
            // get the flow element from the id
            var el = FlowElement.getById(v);
            if (!el) {
                throw new Error('Element with id ' + v + ' is not defined!');
            }
            var d3Node = select(this);
            // now loop all listeners
            for (var _i = 0, _a = el.listeners; _i < _a.length; _i++) {
                var listener = _a[_i];
                d3Node.on(listener.event, listener.callback);
            }
        });
        if (!svgElement) {
            throw new Error('svgElement is null!');
        }
        var groupElement = svgElement.querySelector('g');
        if (!groupElement) {
            throw new Error('groupElement is null!');
        }
        svgElement.setAttribute('width', (groupElement.getBoundingClientRect().width + 40).toString());
        svgElement.setAttribute('height', (groupElement.getBoundingClientRect().height + 40).toString());
    };
    return FlowChart;
}());
export { FlowChart };
//# sourceMappingURL=FlowChart.js.map